#!/bin/csh

#-----------------------------------------------------------------------
# C shell environment setup script for Geant4 10.7.4
#
# This script will configure your environment so that any Geant4 supplied
# tools, libraries and data are available in your PATH, library paths etc.
# Data libraries will only be setup if they were installed as part of the
# Geant4 installation.
#
# Source this script to perform the setup.
#
# This script is autogenerated by CMake DO NOT EDIT
#

#-----------------------------------------------------------------------
# Locate directory of self
#
# Self locate script when sourced
# If sourced interactively, we can use $_ as this should be
#
#   source path_to_script_dir/geant4.csh
#
unset g4sls_sourced_dir
unset geant4_envbindir

set ARGS=($_)
if ("$ARGS" != "") then
  if ("$ARGS[2]" =~ */geant4.csh) then
    set g4sls_sourced_dir="`dirname ${ARGS[2]}`"
  endif
endif

if (! $?g4sls_sourced_dir) then
  # Oh great, we were sourced non-interactively. This means that $_
  # won't be set, so we need an external source of information on
  # where the script is located.
  # We obtain this in one of two ways:
  #   1) Current directory:
  #     cd script_dir ; source geant4.csh
  #
  #   2) Supply the directory as an argument to the script:
  #     source script_dir/geant4.csh script_dir
  #
  if ( -e geant4.csh ) then
    set g4sls_sourced_dir="`pwd`"
  else if ( "$1" != "" )  then
    if ( -e ${1}/geant4.csh ) then
      set g4sls_sourced_dir=${1}
    else
      echo "ERROR ${1} does not contain a Geant4 installation"
    endif
  endif
endif

if (! $?g4sls_sourced_dir) then
  echo "ERROR: geant4.csh could NOT self-locate Geant4 installation"
  echo "because it was sourced (i.e. embedded) in another script."
  echo "This is due to limitations of (t)csh but can be worked around by providing"
  echo "the directory where geant4.csh is located"
  echo "to it, either via cd-ing to the directory before sourcing:"
  echo "  cd where_script_is ; source geant4.csh"
  echo "or by supplying the directory as an argument to the script:"
  echo "  source where_script_is/geant4.csh where_script_is"
  echo " "
  exit 1
endif

set geant4_envbindir="`cd ${g4sls_sourced_dir} > /dev/null ; pwd`"


#-----------------------------------------------------------------------
# Setup Geant4 binary and library paths...
#

if ( ! ${?PATH} ) then
  setenv PATH "$geant4_envbindir"
else
  setenv PATH "$geant4_envbindir":${PATH}
endif
      

if ( ! ${?LD_LIBRARY_PATH} ) then
  setenv LD_LIBRARY_PATH "`cd $geant4_envbindir/../lib > /dev/null ; pwd`"
else
  setenv LD_LIBRARY_PATH "`cd $geant4_envbindir/../lib > /dev/null ; pwd`":${LD_LIBRARY_PATH}
endif
      

#-----------------------------------------------------------------------
# Setup Third-party binary and library paths...
# - CLHEP
# - Builtin CLHEP used

# - XercesC

if ( ! ${?LD_LIBRARY_PATH} ) then
  setenv LD_LIBRARY_PATH /usr/lib/x86_64-linux-gnu
else
  setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:/usr/lib/x86_64-linux-gnu
endif
      

#-----------------------------------------------------------------------
# Resource file paths
# - Datasets
if [ ! {G4NEUTRONHPDATA} ] ; then setenv G4NEUTRONHPDATA "/opt/geant4/geant4-data/G4NDL4.6" ; fi
if [ ! {G4LEDATA} ] ; then setenv G4LEDATA "/opt/geant4/geant4-data/G4EMLOW7.13" ; fi
if [ ! {G4LEVELGAMMADATA} ] ; then setenv G4LEVELGAMMADATA "/opt/geant4/geant4-data/PhotonEvaporation5.7" ; fi
if [ ! {G4RADIOACTIVEDATA} ] ; then setenv G4RADIOACTIVEDATA "/opt/geant4/geant4-data/RadioactiveDecay5.6" ; fi
if [ ! {G4PARTICLEXSDATA} ] ; then setenv G4PARTICLEXSDATA "/opt/geant4/geant4-data/G4PARTICLEXS3.1.1" ; fi
if [ ! {G4PIIDATA} ] ; then setenv G4PIIDATA "/opt/geant4/geant4-data/G4PII1.3" ; fi
if [ ! {G4REALSURFACEDATA} ] ; then setenv G4REALSURFACEDATA "/opt/geant4/geant4-data/RealSurface2.2" ; fi
if [ ! {G4SAIDXSDATA} ] ; then setenv G4SAIDXSDATA "/opt/geant4/geant4-data/G4SAIDDATA2.0" ; fi
if [ ! {G4ABLADATA} ] ; then setenv G4ABLADATA "/opt/geant4/geant4-data/G4ABLA3.1" ; fi
if [ ! {G4INCLDATA} ] ; then setenv G4INCLDATA "/opt/geant4/geant4-data/G4INCL1.0" ; fi
if [ ! {G4ENSDFSTATEDATA} ] ; then setenv G4ENSDFSTATEDATA "/opt/geant4/geant4-data/G4ENSDFSTATE2.3" ; fi


# - Fonts for Freetype

if ( ! ${?TOOLS_FONT_PATH} ) then
  setenv TOOLS_FONT_PATH "`cd $geant4_envbindir/../share/Geant4-10.7.4/fonts > /dev/null ; pwd`"
else
  setenv TOOLS_FONT_PATH ${TOOLS_FONT_PATH}:"`cd $geant4_envbindir/../share/Geant4-10.7.4/fonts > /dev/null ; pwd`"
endif
      

#----------------------------------------------------------------------

